package com.yilin.xbr.xbr_gaode_track;

import android.app.Activity;

import androidx.annotation.NonNull;

import java.util.Map;

import io.flutter.embedding.engine.plugins.FlutterPlugin;
import io.flutter.embedding.engine.plugins.activity.ActivityAware;
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;
import io.flutter.plugin.common.MethodChannel.Result;

/**
 * XbrGaodeTrackPlugin
 */
public class XbrGaodeTrackPlugin implements FlutterPlugin, MethodCallHandler, ActivityAware {

    private GaodeTrackService gaodeTrackService;
    private MethodChannel channel;
    private Activity activity;


    public GaodeTrackService getGaodeTrackService() {
        if (gaodeTrackService == null) gaodeTrackService = new GaodeTrackService();
        return gaodeTrackService;
    }

    @Override
    public void onAttachedToEngine(@NonNull FlutterPluginBinding flutterPluginBinding) {
        channel = new MethodChannel(flutterPluginBinding.getBinaryMessenger(), "xbr_gaode_track");
        channel.setMethodCallHandler(this);
    }

    @Override
    public void onMethodCall(@NonNull MethodCall call, @NonNull Result result) {
        switch (call.method) {
            case "initGaodeTrackSDK":{
                String serviceId = null;Integer gatherInterval = 0, uploadInterval = 0, cacheSize = 0, locationMode = 0;
                if (call.hasArgument("serviceId")) serviceId = call.argument("serviceId");
                if (call.hasArgument("gatherInterval")) gatherInterval = call.argument("gatherInterval");
                if (call.hasArgument("uploadInterval")) uploadInterval = call.argument("uploadInterval");
                if (call.hasArgument("cacheSize")) cacheSize = call.argument("cacheSize");
                if (call.hasArgument("locationMode")) locationMode = call.argument("locationMode");
                getGaodeTrackService().initGaodeTrackSDK(activity,serviceId, gatherInterval, uploadInterval, cacheSize, locationMode);
                result.success(toMapJson(true, "成功"));
                break;
            }
            case "addTerminal": {
                //######## 添加终端
                String terminalName = null;
                if (call.hasArgument("terminalName")) terminalName = call.argument("terminalName");
                if (terminalName == null) result.success(toMapJson(false, "添加终端失败，参数不完整！"));
                getGaodeTrackService().addTerminal(terminalName, toResult(result));
                break;
            }
            case "addTrack": {
                //######## 添加轨迹
                String terminalId = null;
                if (call.hasArgument("terminalId")) terminalId = call.argument("terminalId");
                if (terminalId == null) result.success(toMapJson(false, "添加轨迹失败，参数不完整！"));
                getGaodeTrackService().addTrack(terminalId, toResult(result));
                break;
            }
            case "startTrack": {
                //######## 开始轨迹上传
                String terminalId = null, trackId = null;
                if (call.hasArgument("terminalId")) terminalId = call.argument("terminalId");
                if (call.hasArgument("trackId")) trackId = call.argument("trackId");
                if (terminalId == null || trackId == null) result.success(toMapJson(false, "开始轨迹失败，参数不完整！"));
                getGaodeTrackService().startTrack(terminalId, trackId, toResult(result));
                break;
            }
            case "startGather":
                //######## 开始采集
                getGaodeTrackService().startGather(toResult(result));
                break;
            case "stopGather":
                //######## 停止采集
                getGaodeTrackService().stopGather(toResult(result));
                break;
            case "stopTrack": {
                //######## 停止轨迹上传
                getGaodeTrackService().stopTrack(toResult(result));
                break;
            }
            case "queryLatestPoint": {
                //######## 查询当前位置
                String serviceId = null, terminalId = null;
                if (call.hasArgument("serviceId")) serviceId = call.argument("serviceId");
                if (call.hasArgument("terminalId")) terminalId = call.argument("terminalId");
                if (serviceId == null || terminalId == null) result.success(toMapJson(false, "查询位置失败，参数不完整！"));
                getGaodeTrackService().queryLatestPoint(serviceId, terminalId, toResult(result));
                break;
            }
            case "queryTerminalTrack": {
                //######## 查存轨迹
                String serviceId = null, terminalId = null, trackId = null;
                Long startTime = null, endTime = null;
                Integer noising = null, bindLoad = null, accFilter = null, driveMode = null, disCompensation = null,
                    compensationMeter = null, includePoints = null, page = null, limit = null;
                if (call.hasArgument("serviceId")) serviceId = call.argument("serviceId");
                if (call.hasArgument("terminalId")) terminalId = call.argument("terminalId");
                if (call.hasArgument("trackId")) trackId = call.argument("trackId");
                if (call.hasArgument("startTime")) startTime = call.argument("startTime");
                if (call.hasArgument("endTime")) endTime = call.argument("endTime");
                if (call.hasArgument("noising")) noising = call.argument("noising");
                if (call.hasArgument("bindLoad")) bindLoad = call.argument("bindLoad");
                if (call.hasArgument("accFilter")) accFilter = call.argument("accFilter");
                if (call.hasArgument("driveMode")) driveMode = call.argument("driveMode");
                if (call.hasArgument("disCompensation")) disCompensation = call.argument("disCompensation");
                if (call.hasArgument("compensationMeter")) compensationMeter = call.argument("compensationMeter");
                if (call.hasArgument("includePoints")) includePoints = call.argument("includePoints");
                if (call.hasArgument("page")) page = call.argument("page");
                if (call.hasArgument("limit")) limit = call.argument("limit");
                if (serviceId == null || terminalId == null || trackId == null) result.success(toMapJson(false, "查询位置失败，参数不完整！"));
                getGaodeTrackService().queryTerminalTrack(serviceId, terminalId, trackId, startTime, endTime, noising, bindLoad,
                    accFilter, driveMode, disCompensation, compensationMeter, includePoints, page, limit, toResult(result));
                break;
            }
            default:
                result.notImplemented();
                break;
        }
    }

    @Override
    public void onDetachedFromEngine(@NonNull FlutterPluginBinding binding) {
        channel.setMethodCallHandler(null);
    }

    @Override
    public void onAttachedToActivity(@NonNull ActivityPluginBinding binding) {
        activity = binding.getActivity();
    }

    @Override
    public void onDetachedFromActivityForConfigChanges() {

    }

    @Override
    public void onReattachedToActivityForConfigChanges(@NonNull ActivityPluginBinding binding) {
        onAttachedToActivity(binding);
    }

    @Override
    public void onDetachedFromActivity() {

    }


    public String toMapJson(Boolean success, String msg) {
        return toMapJson(success ? 0 : -1, success, msg, null);
    }

    public <T> GaodeTrackService.Callback<T> toResult(@NonNull Result result) {
        return (code, success, msg, object) -> result.success(toMapJson(code, success, msg, object));
    }

    public String toMapJson(int code, Boolean success, String msg, Object data) {
        Map<String, Object> json = MapUtil.generate(map -> {
            map.put("code", code);
            map.put("success", success);
            map.put("msg", msg);
            map.put("data", data);
        });
        return GsonUtil.toJson(json);
    }

}
