#import "XbrGaodeTrackPlugin.h"
#import <objc/runtime.h>

@implementation XbrGaodeTrackPlugin
+ (void)registerWithRegistrar:(NSObject<FlutterPluginRegistrar>*)registrar {
    FlutterMethodChannel* channel = [FlutterMethodChannel
                                     methodChannelWithName:@"xbr_gaode_track"
                                     binaryMessenger:[registrar messenger]];
    XbrGaodeTrackPlugin* instance = [[XbrGaodeTrackPlugin alloc] init];
    [registrar addMethodCallDelegate:instance channel:channel];
}

- (void)handleMethodCall:(FlutterMethodCall*)call result:(FlutterResult)result {
    NSDictionary<NSString*, id>* args = call.arguments;
    self.result = result;
    if ([@"getPlatformVersion" isEqualToString:call.method]) {
        result([@"iOS " stringByAppendingString:[[UIDevice currentDevice] systemVersion]]);
    }else if([@"initGaodeTrackSDK" isEqualToString:call.method]){
        NSString* apiKey = args[@"apiKey"];
        NSString* serviceId  = args[@"serviceId "];
        NSNumber* gatherInterval = args[@"gatherInterval"];
        NSNumber* uploadInterval = args[@"uploadInterval"];
        NSNumber* cacheSize = args[@"cacheSize"];
        NSNumber* locationMode = args[@"locationMode"];
        [AMapServices sharedServices].apiKey = apiKey;
        self.option = [[AMapTrackManagerOptions alloc] init];
        self.option.serviceID = serviceId; //Service ID 需要根据需要进行修改
        self.trackManager = [[AMapTrackManager alloc] initWithOptions:self.option];
        self.trackManager.delegate = (id)self;
        [self.trackManager setAllowsBackgroundLocationUpdates:YES];
        [self.trackManager setPausesLocationUpdatesAutomatically:NO];
        [self.trackManager changeGatherAndPackTimeInterval:5 packTimeInterval:30];
        [self.trackManager setLocalCacheMaxSize:50];
        [self toResult: @{@"code":@0, @"success":@YES }];
        
    }else if([@"addTerminal" isEqualToString:call.method]){
        NSString* terminalName  = args[@"terminalName"];
        //创建终端
        AMapTrackAddTerminalRequest *addRequest = [[AMapTrackAddTerminalRequest alloc] init];
        addRequest.serviceID = self.trackManager.serviceID;
        addRequest.terminalName = terminalName;
        [self.trackManager AMapTrackAddTerminal:addRequest];
        
    }else if([@"addTrack" isEqualToString:call.method]){
        NSString* terminalId  = args[@"terminalId"];
        //创建轨迹
        AMapTrackAddTrackRequest *request = [[AMapTrackAddTrackRequest alloc] init];
        request.serviceID = self.trackManager.serviceID;
        request.terminalID = terminalId;
        [self.trackManager AMapTrackAddTrack:request];
        
    }else if([@"startTrack" isEqualToString:call.method]){
        NSString* terminalId  = args[@"terminalId"];
        NSString* trackId  = args[@"trackId"];
        //开始上报服务
        AMapTrackManagerServiceOption *serviceOption = [[AMapTrackManagerServiceOption alloc] init];
        serviceOption.terminalID = terminalId;//Terminal ID 需要根据需要进行修改
        self.trackManager.trackID = trackId;
        [self.trackManager startServiceWithOptions:serviceOption];
        
    }else if([@"startGather" isEqualToString:call.method]){
        //开始采集
        [self.trackManager startGatherAndPack];
        
    }else if([@"stopGather" isEqualToString:call.method]){
        //停止采集
        [self.trackManager stopGaterAndPack];
        
    }else if([@"stopTrack" isEqualToString:call.method]){
        //停止上报服务
        [self.trackManager stopService];
        
    }else if([@"queryLatestPoint" isEqualToString:call.method]){
        NSString* serviceId   = args[@"serviceId"];
        NSString* terminalId  = args[@"terminalId"];
        
        AMapTrackQueryLastPointRequest *request = [[AMapTrackQueryLastPointRequest alloc] init];
        request.serviceID = serviceId;
        request.terminalID = terminalId;
        [self.trackManager AMapTrackQueryLastPoint:request];
        
    }else if([@"queryTerminalTrack" isEqualToString:call.method]){
        NSString* serviceId   = args[@"serviceId"];
        NSString* terminalId  = args[@"terminalId"];
        NSString* trackId  = args[@"trackId"];
        NSNumber* startTime  = args[@"startTime"];
        NSNumber* endTime  = args[@"endTime"];
        NSNumber* page  = args[@"page"];
        NSNumber* limit  = args[@"limit"];
        
        NSNumber* noising  = args[@"noising"];//ios没有定义 去噪
        NSNumber* bindLoad  = args[@"bindLoad"];
        NSNumber* accFilter   = args[@"accFilter"];//ios没有定义 精度过滤
        NSNumber* driveMode  = args[@"driveMode"];
        NSNumber* disCompensation  = args[@"disCompensation"];
        NSNumber* compensationMeter  = args[@"compensationMeter"];
        NSNumber* includePoints   = args[@"includePoints"];
        
        if(startTime==nil) startTime = [NSNumber numberWithLong:([[NSDate date] timeIntervalSince1970] - 12*60*60) * 1000] ;
        if(endTime==nil) endTime = [NSNumber numberWithLong:[[NSDate date] timeIntervalSince1970] * 1000] ;
        
        AMapTrackQueryTrackInfoRequest *request = [[AMapTrackQueryTrackInfoRequest alloc] init];
        request.serviceID = serviceId;
        request.terminalID = terminalId;
        request.trackID = trackId;
        request.startTime = startTime.longValue;
        request.endTime = endTime.longValue;
        
        request.pageIndex = page.intValue;
        request.pageSize = endTime.intValue;
        
        request.recoupMode = disCompensation.intValue;//补点模式
        request.recoupGap = compensationMeter.intValue;//补点 5000
        request.correctionMode = bindLoad.stringValue;//纠偏
        request.containPoints = includePoints.intValue;//包含点位
        
        [self.trackManager AMapTrackQueryTrackInfo:request];
        
    }else {
        result(FlutterMethodNotImplemented);
    }
}

// 回调接口区

//开始上报回调
- (void)onStartService:(AMapTrackErrorCode)errorCode {
    NSNumber* code = [NSNumber numberWithLong:errorCode];
    if (errorCode == AMapTrackErrorOK) {
        [self toResult: @{@"code":code, @"success":@YES}];
    } else {
        [self toResult: @{@"code":code, @"success":@NO, @"msg":@"开始服务失败，具体查看错误码"}];
    }
}

//开始采集回调
- (void)onStartGatherAndPack:(AMapTrackErrorCode)errorCode {
    NSNumber* code = [NSNumber numberWithLong:errorCode];
    if (errorCode == AMapTrackErrorOK) {
        [self toResult: @{@"code":code, @"success":@YES}];
    } else {
        [self toResult: @{@"code":code, @"success":@NO, @"msg":@"开始采集失败，具体查看错误码"}];
    }
}

//停止服务回调
- (void)onStopService:(AMapTrackErrorCode)errorCode{
    NSNumber* code = [NSNumber numberWithLong:errorCode];
    if (errorCode == AMapTrackErrorOK) {
        [self toResult: @{@"code":code, @"success":@YES}];
    } else {
        [self toResult: @{@"code":code, @"success":@NO, @"msg":@"停止服务失败，具体查看错误码"}];
    }
}

//停止采集回调
- (void)onStopGatherAndPack:(AMapTrackErrorCode)errorCode{
    NSNumber* code = [NSNumber numberWithLong:errorCode];
    if (errorCode == AMapTrackErrorOK) {
        [self toResult: @{@"code":code, @"success":@YES}];
    } else {
        [self toResult: @{@"code":code, @"success":@NO, @"msg":@"停止采集失败，具体查看错误码"}];
    }
}

//创建终端结果
- (void)onAddTerminalDone:(AMapTrackAddTerminalRequest *)request response:(AMapTrackAddTerminalResponse *)response {
    //创建terminal成功
    NSString *terminalID = [response terminalID];
    [self toResult: @{@"code":@0, @"success":@YES, @"data":terminalID}];
}

//创建轨迹结果
- (void)onAddTrackDone:(AMapTrackAddTrackRequest *)request response:(AMapTrackAddTrackResponse *)response {
    //创建轨迹成功，开始采集
    NSString * trackID = response.trackID;
    [self toResult: @{@"code":@0, @"success":@YES, @"data":trackID}];
}

- (void)onQueryLastPointDone:(AMapTrackQueryLastPointRequest *)request response:(AMapTrackQueryLastPointResponse *)response{
   //查询成功
    [self toResult: @{@"code":@0, @"success":@YES, @"data":response.formattedDescription}];
}

- (void)onQueryTrackInfoDone:(AMapTrackQueryTrackInfoRequest *)request response:(AMapTrackQueryTrackInfoResponse *)response {
    //查询成功
    [self toResult: @{@"code":@0, @"success":@YES, @"data":response.formattedDescription}];
}


//错误回调
- (void)didFailWithError:(NSError *)error associatedRequest:(id)request {
    NSNumber* code = [NSNumber numberWithLong:[error code]];
    NSString* msg = [error domain];
    // 为方便以后修改 下面暂时不合并
    if ([request isKindOfClass:[AMapTrackQueryTerminalRequest class]]) {
       //查询参数错误
        [self toResult: @{@"code": code, @"success":@NO,@"msg":msg}];
    }
   
    if ([request isKindOfClass:[AMapTrackAddTerminalRequest class]]) {
       //创建terminal失败
        [self toResult: @{@"code": code, @"success":@NO,@"msg":msg}];
    }
    
    if ([request isKindOfClass:[AMapTrackAddTrackRequest class]]) {
        //创建轨迹失败
        [self toResult: @{@"code": code, @"success":@NO,@"msg":msg}];
    }

    if ([request isKindOfClass:[AMapTrackQueryLastPointRequest class]]) {
       //查询失败
        [self toResult: @{@"code": code, @"success":@NO,@"msg":msg}];
    }
    
    if ([request isKindOfClass:[AMapTrackQueryTrackInfoRequest class]]) {
        //查询失败
        [self toResult: @{@"code": code, @"success":@NO,@"msg":msg}];
    }
}

- (void)toResult:(NSDictionary*)map{
    //转 json 字符串
    NSData* jsonData =[NSJSONSerialization dataWithJSONObject:map options:NSJSONWritingPrettyPrinted error:nil];
    NSString* jsonStr =[[NSString alloc] initWithData:jsonData encoding:NSUTF8StringEncoding];
    if (self.result != nil) self.result(jsonStr);
}


// 公共方法

//转NSMutableDictionary
+ (NSDictionary*)getObjectData:(id)obj{
    NSMutableDictionary *dic = [NSMutableDictionary dictionary];
    unsigned int propsCount;
    objc_property_t *props = class_copyPropertyList([obj class], &propsCount);//获得属性列表
    for(int i = 0;i < propsCount; i++){
        objc_property_t prop = props[i];
        NSString *propName = [NSString stringWithUTF8String:property_getName(prop)];//获得属性的名称
        id value = [obj valueForKey:propName];//kvc读值
        if(value == nil){
            value = [NSNull null];
        }else{
            value = [self getObjectInternal:value];//自定义处理数组，字典，其他类
        }
        [dic setObject:value forKey:propName];
    }
    return dic;
}
 

+ (id)getObjectInternal:(id)obj{
    if([obj isKindOfClass:[NSString class]]
       || [obj isKindOfClass:[NSNumber class]]
       || [obj isKindOfClass:[NSNull class]])
    {
        return obj;
    }
    
    if([obj isKindOfClass:[NSArray class]])
    {
        NSArray *objarr = obj;
        NSMutableArray *arr = [NSMutableArray arrayWithCapacity:objarr.count];
        for(int i = 0;i < objarr.count; i++)
        {
            [arr setObject:[self getObjectInternal:[objarr objectAtIndex:i]] atIndexedSubscript:i];
        }
        return arr;
    }
    
    if([obj isKindOfClass:[NSDictionary class]])
    {
        NSDictionary *objdic = obj;
        NSMutableDictionary *dic = [NSMutableDictionary dictionaryWithCapacity:[objdic count]];
        for(NSString *key in objdic.allKeys)
        {
            [dic setObject:[self getObjectInternal:[objdic objectForKey:key]] forKey:key];
        }
        return dic;
    }
    return [self getObjectData:obj];
}

@end
